#!/bin/sh
###########################################
#                                         #
#  MinimServer Synology package           #
#                                         #
###########################################

TMP_DIR="/tmp/MinimServer"
DEBUG_FILE="${TMP_DIR}/minimserver-debug"
if [ -f "$DEBUG_FILE" ]; then
	exec >>$DEBUG_FILE 2>&1
	set -x
fi
APP_DATA="appData"
LOG_FILE="${TMP_DIR}/minimserver-log"
PACKAGE_LOCATION="/var/packages/${SYNOPKG_PKGNAME}/target"
if [ -f "/etc/samba/smb.conf" ]; then
	# location for DSM 6.0 and later
	smb_conf="/etc/samba/smb.conf"
elif [ -f "/usr/syno/etc/smb.conf" ]; then
	# location for DSM 5.2 and earlier
	smb_conf="/usr/syno/etc/smb.conf"
fi
if [ -n "$smb_conf" ]; then
	minim_share=$(cat "$smb_conf" | sed -r '/\/'MinimServer'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'MinimServer').*$/\1/')
fi
if [ -n "$minim_share" ]; then
	# shared folder already exists and might not be on the package installed volume
	VOLUME=$(dirname "$minim_share")
	MINIM_HOME="$minim_share"
else
	appstore_target=$(readlink "${PACKAGE_LOCATION}")
	if [ ! -z "$appstore_target" ]; then
		appstore_dir=$(dirname "$appstore_target")
		VOLUME=$(dirname "$appstore_dir")
	else
		VOLUME=$(/usr/syno/bin/servicetool --get-alive-volume)
	fi
	MINIM_HOME="${VOLUME}/${SYNOPKG_PKGNAME}"
fi
[ ! -e "${TMP_DIR}" ] && mkdir "${TMP_DIR}"

debug ()
{
	DATE=$(date +"%a %d %b %T")
	echo "${DATE} $1" >>$LOG_FILE
}

is_dsm7 ()
{
	MAJOR=$(grep majorversion /etc/VERSION | cut -f2 -d'"')
	[ $MAJOR -ge 7 ]
}

##################################################################################

preinst ()
{
	exit 0
}

##################################################################################

postinst ()
{
	# create home directory if it doesn't exist
	if ! is_dsm7; then
		# not DSM7: create MinimServer shared folder
		if [ ! -d "${MINIM_HOME}" ]; then
			synoshare --add "${SYNOPKG_PKGNAME}" "${SYNOPKG_PKGNAME} configuration" "${MINIM_HOME}" "" "@administrators" "@users" 0 0
			synoacltool -add "${MINIM_HOME}" everyone:*:allow:r-x---a-R-c--:-d-- >/dev/null
			synoacltool -add "${MINIM_HOME}" everyone:*:allow:r-----a-R-c--:f--- >/dev/null
		fi
	fi
	if [ ! -d "$MINIM_HOME/$APP_DATA" ]; then
		mkdir "$MINIM_HOME/$APP_DATA"
	fi
	if [ ! -d "$MINIM_HOME/etc" ]; then
		mkdir "$MINIM_HOME/etc"
	fi
	if [ ! -d "$MINIM_HOME/opt" ]; then
		mkdir "$MINIM_HOME/opt"
	fi
	if [ ! -d "$MINIM_HOME/opt/bin" ]; then
		mkdir "$MINIM_HOME/opt/bin"
	fi

	# restore the backup config file if it exists
	if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -f "$MINIM_HOME/backup/minimserver.config" ]; then
		backup_config_dir="$MINIM_HOME/backup"
	else
		# need to check the public share if updating from a version of MinimServer prior to 0.8.4.1
		if [ -f "/etc/samba/smb.conf" ]; then
			# location for DSM 6.0 and later
			smb_conf="/etc/samba/smb.conf"
		elif [ -f "/usr/syno/etc/smb.conf" ]; then
			# location for DSM 5.2 and earlier
			smb_conf="/usr/syno/etc/smb.conf"
		fi
		if [ -n "$smb_conf" ]; then
			public_dir=$(cat "$smb_conf" | sed -r '/\/'public'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'public').*$/\1/')
			if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -n "$public_dir" -a -f "${public_dir}/minimserver/minimserver.config" ]; then
				backup_config_dir="${public_dir}/minimserver"
			fi
		fi
	fi
	if [ -n "${backup_config_dir}" -a -f "${backup_config_dir}/minimserver.config" ]; then
		# use the backup config file (takes priority over contents of appData)
		cp -p "${backup_config_dir}/minimserver.config" "$MINIM_HOME/$APP_DATA"
		# remove the configuration backup directory
		rm -r "${backup_config_dir}"
	fi

	if ! is_dsm7; then
		# if no conf/resource file or upgrading from 0.8.x, need to install the service configure file
		if [ ! -f /usr/local/etc/services.d/MinimServer.sc ]; then
			/usr/syno/bin/servicetool --install-configure-file --package ${PACKAGE_LOCATION}/etc/MinimServer.sc >/dev/null
		fi
	fi

	exit 0
}

##################################################################################

preuninst ()
{
	debug "preuninst: sps=${SYNOPKG_PKG_STATUS}"

	# if uninstalling, check for full license still active
	if [ "${SYNOPKG_PKG_STATUS}" == "UNINSTALL" ]; then
		grep -q ".licenseType = full" ${MINIM_HOME}/${APP_DATA}/minimserver.config
		is_full_license=$?
		debug "preuninst: is_full_license=$is_full_license"
		if [ $is_full_license -eq 0 ]; then
			grep -q "minimserver.deactivateLicenseError = true" ${MINIM_HOME}/${APP_DATA}/minimserver.config
			is_deactivate_error=$?
			debug "preuninst: is_deactivate_error=$is_deactivate_error"
			if [ $is_deactivate_error -ne 0 ]; then
				echo "You must deactivate your MinimServer full license before uninstalling." >$SYNOPKG_TEMP_LOGFILE
				exit 1
			fi
		fi
	fi

	# if upgrading, make a backup of MinimServer application data and other directories
	if [ "${SYNOPKG_PKG_STATUS}" == "UPGRADE" ]; then
		if [ ! -d "${VOLUME}/@tmp/minimtmp" ]; then
			mkdir ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${MINIM_HOME}/${APP_DATA}" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/${APP_DATA}" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/${APP_DATA}"
			fi
			cp -pr ${MINIM_HOME}/${APP_DATA} ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${MINIM_HOME}/opt" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/opt" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/opt"
			fi
			cp -pr ${MINIM_HOME}/opt ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${PACKAGE_LOCATION}/libext" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/libext" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/libext"
			fi
			cp -pr ${PACKAGE_LOCATION}/libext ${VOLUME}/@tmp/minimtmp
		fi
		if ! is_dsm7; then
			if [ -d "${MINIM_HOME}/etc" ]; then
				# if backup already done by preupgrade, replace it
				if [ -d "${VOLUME}/@tmp/minimtmp/etc" ]; then
					rm -r "${VOLUME}/@tmp/minimtmp/etc"
				fi
				cp -pr ${MINIM_HOME}/etc ${VOLUME}/@tmp/minimtmp
			fi
		fi
	fi

	# clean up if uninstalling
	if [ "${SYNOPKG_PKG_STATUS}" == "UNINSTALL" ]; then
		# backup minimserver.config if chosen in the uninstallation dialog
		backup_dir="${MINIM_HOME}/backup"
		if [ "$backup_key" == "true" ]; then
			# copy the configuration file to the backup directory
			[ ! -d "${backup_dir}" ] && mkdir "${backup_dir}"
			cp -p "${MINIM_HOME}/${APP_DATA}/minimserver.config" "${backup_dir}"
			# (new) leave everything in MINIM_HOME unchanged
		elif [ "$backup_key" == "false" ]; then
			# remove the configuration backup directory if it exists
			if [ -d "${backup_dir}" ]; then
				rm -r "${backup_dir}"
			fi

			# remove appData directory
			if [ -d "${MINIM_HOME}/${APP_DATA}" ]; then
				rm -r "${MINIM_HOME}/${APP_DATA}"
			fi

			# remove opt and etc directories (previously not done for DSM7)
			if [ -d "${MINIM_HOME}/opt" ]; then
				rm -r "${MINIM_HOME}/opt"
			fi
			if [ -d "${MINIM_HOME}/etc" ]; then
				ETC_FILES=$(ls "${MINIM_HOME}/etc"/ServerBundle_*.defaults 2>/dev/null)
				[ -n "${ETC_FILES}" ] && rm -f "${MINIM_HOME}"/etc/ServerBundle_*.defaults
				# remove profiles backup directory and its contents
				rm -rf "${MINIM_HOME}"/etc/profiles
				# remove recent directory and its contents
				rm -rf "${MINIM_HOME}"/etc/recent
				# don't remove etc directory if it contains user-created files
				[ -z "$(ls -A "${MINIM_HOME}/etc")" ] && rmdir "${MINIM_HOME}/etc"
			fi

			# remove license files
			rm /usr/syno/etc/packages/MinimServer/*.license

			if ! is_dsm7; then
				# remove home directory unless it contains a configuration backup or user-created files
				if [ "$(ls -A "${MINIM_HOME}")" = @eaDir ]; then
					rm -rf "${MINIM_HOME}/@eaDir"
				fi
				[ -z "$(ls -A "${MINIM_HOME}")" ] && synoshare --del TRUE "${SYNOPKG_PKGNAME}"

			fi
		fi

		if ! is_dsm7; then
			# if no conf/resource file, uninstall the service configure file
			if [ ! -f /var/packages/${SYNOPKG_PKGNAME}/conf/resource ]; then
				/usr/syno/bin/servicetool --remove-configure-file --package MinimServer.sc >/dev/null
			fi
		fi
	fi

	exit 0
}

##################################################################################

postuninst ()
{
	exit 0
}

##################################################################################

preupgrade ()
{
	grep -q "\\.licenseType = full" ${MINIM_HOME}/${APP_DATA}/minimserver.config
	is_full_license=$?
	debug "updates_check: is_full_license=$is_full_license"
	if [ $is_full_license -eq 0 ]; then
		expiry_date=$(grep "\\.updatesExpiry" ${MINIM_HOME}/${APP_DATA}/minimserver.config | cut -f2- -d'=' | sed 's/ //g')
		package_version=$(grep "version=" $SYNOPKG_PKGINST_TEMP_DIR/../INFO | cut -f2 -d'"' | cut -f1 -d'-')
		package_date=$(grep "\\.packageDate" $SYNOPKG_PKGINST_TEMP_DIR/lib/minimserver-$package_version.jmpkg | cut -f2- -d'=' | sed 's/ //g')
		debug "updates_check: expiry_date=$expiry_date package_version=$package_version package_date=$package_date"
		if [ -n "${expiry_date##*[!0-9]*}" -a $expiry_date -lt $package_date ]; then
			echo "Your updates entitlement for your MinimServer full license expired on ${expiry_date:0:4}-${expiry_date:4:2}-${expiry_date:6}." \
				"To install MinimServer $package_version, you need to purchase an updates extension license." \
				"For more information, visit minimserver.com/updates.html." >$SYNOPKG_TEMP_LOGFILE
			exit 1
		fi
	fi

	exit 0
}

##################################################################################

postupgrade ()
{
	# restore the backup of MinimServer application data and other directories
	if [ -d "${VOLUME}/@tmp/minimtmp" ]; then
		if [ -d "${VOLUME}/@tmp/minimtmp/${APP_DATA}" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/${APP_DATA} ${MINIM_HOME}
			if ! is_dsm7; then
				# the files in appData might have been created by the daemon user
				chown -R "root:root" ${MINIM_HOME}/${APP_DATA}
			fi
		fi
		if [ -d "${VOLUME}/@tmp/minimtmp/opt" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/opt ${MINIM_HOME}
		fi
		if [ -d "${VOLUME}/@tmp/minimtmp/libext" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/libext ${PACKAGE_LOCATION}
			if ! is_dsm7; then
				# the files in libext might have been created by the daemon user
				chown -R "root:root" ${VOLUME}/@tmp/minimtmp/libext
			fi
		fi
		if ! is_dsm7; then
			if [ -d "${VOLUME}/@tmp/minimtmp/etc" ]; then
				cp -pr ${VOLUME}/@tmp/minimtmp/etc ${MINIM_HOME}
			fi
		fi
		rm -r ${VOLUME}/@tmp/minimtmp
	fi

	exit 0
}

##################################################################################
